name = 'basetestcase'

from .base_FormTestCase import FormTestCase
from .base_FunctionalTestCase import FunctionalTestCase
from .base_ModelTestCase import ModelTestCase
from .base_ViewTestCase import ViewTestCase